CQ.Smartling.DictionaryTargetInfoPanel = CQ.Ext.extend(CQ.form.CompositeField, {
    cqLocalePath: null,
    smartlingLocale: null,

    constructor: function (config) {
        config = config || {};

        var defaults = {
            "labelWidth": 50,
            "labelPad": 0,
            "layout": "fit",
            "padding": 10
        };

        config = CQ.Util.applyDefaults(config, defaults);
        CQ.Smartling.ContentTargetInfoPanel.superclass.constructor.call(this, config);
    },

    initComponent: function () {
        CQ.Smartling.DictionaryTargetInfoPanel.superclass.initComponent.call(this);

        dictionaryTargetInfoPanel = new CQ.form.CompositeField({
            layout: "form",
            border: false,
            autoHeight: true
        });

        this.cqLocalePath = new CQ.form.Selection({
            fieldLabel: "Adobe CQ Locale",
            name: "cqLocalePath",
            width: 190,
            type: "select",
            options: "/services/translation/cqlocales",
            optionsRoot: "cqLocales",
            optionsTextField: "label",
            optionsValueField: "path",
            allowBlank: false,
            msgTarget: "qtip"
        });
        this.cqLocalePath.setValue("");
        dictionaryTargetInfoPanel.add(this.cqLocalePath);

        this.smartlingLocale = new CQ.Ext.form.TextField({
            fieldLabel: "Smartling Locale",
            name: "smartlingLocale",
            allowBlank: false,
            blankText: "Smartling locale is required.",
            width: 190
        });
        dictionaryTargetInfoPanel.add(this.smartlingLocale);
        this.add(dictionaryTargetInfoPanel);
    },

    processInit: function (path, record) {
        this.cqLocalePath.processInit(path, record);
        this.smartlingLocale.processInit(path, record);
    },

    setValue: function (value) {
        var targetInfo = JSON.parse(value);
        this.cqLocalePath.setValue(targetInfo.cqLocalePath);
        this.smartlingLocale.setValue(targetInfo.smartlingLocale);
        this.validate();
    },

    getValue: function () {
        return this.getRawValue();
    },

    getRawValue: function () {
        var targetInfo = {
            "cqLocalePath": this.cqLocalePath.getValue(),
            "smartlingLocale": this.smartlingLocale.getValue()
        };
        return JSON.stringify(targetInfo);
    }
});
CQ.Ext.reg("dictionarytargetinfo", CQ.Smartling.DictionaryTargetInfoPanel);

CQ.Smartling.DictionaryMappingDialog = CQ.Ext.extend(CQ.Smartling.Dialog, {
    form: null,

    getTranslationConfigsPanel: function() {
        return {
               title: "Adobe CQ settings",
               items: [
                   {
                       xtype: "fieldset",
                       autoScroll: true,
                       defaults: {
                           xtype: "textfield",
                           width: 375
                       },
                       items: [
                           {
                               fieldLabel: "Source Path",
                               id: "dictionarySourcePath",
                               ref: "../sourcePath",
                               name: "sourcePath",
                               value: this.sourcePath,
                               rootPath: "/",
                               selectOnFocus: "true",
                               editable: false,
                               allowBlank: false,
                               regex: /\/.+/,
                               regexText: "Source path shouldn't be root",
                               xtype: "pathfield"
                           },
                           {
                               fieldLabel: "Source Locale",
                               id: "dictionarySourceLocalePath",
                               ref: "../sourceLocalePath",
                               name: "sourceLocalePath",
                               options: "/services/translation/cqlocales",
                               optionsRoot: "cqLocales",
                               optionsTextField: "label",
                               optionsValueField: "path",
                               allowBlank: false,
                               msgTarget: "qtip",
                               type: "select",
                               xtype: "selection"
                           },
                           {
                               fieldLabel: "Target Info",
                               name: "dictionaryTargetInfoPanel",
                               ref: "../dictionaryTargetInfoPanel",
                               autoScroll: true,
                               height: 370,
                               padding: "5px 5px 0 5px",
                               xtype: "panel",
                               items: [
                                   {
                                       name: "dictionaryTargetInfo",
                                       ref: "../dictionaryTargetInfo",
                                       orderable: false,
                                       width: 350,
                                       fieldConfig: {
                                           xtype: "dictionarytargetinfo"
                                       },
                                       xtype: "multifield"
                                   }
                               ]
                           }
                       ]
                   }
               ]
           }
    },

    constructor: function (config) {
        CQ.Ext.applyIf(config, {
            formCfg: {
                labelWidth: 80,
                labelPad: 0,
                defaults: {
                    padding: 15,
                    width: 520,
                    height: 515
                }
            },
            items: [
                {
                    xtype: "tabpanel",
                    activeTab: 0,
                    items: [
                        {
                            title: "General settings",
                            items: [
                                {
                                    xtype: "fieldset",
                                    height: 455,
                                    defaults: {
                                        xtype: "textfield",
                                        width: 375
                                    },
                                    items: [
                                        {
                                            ref: "../configId",
                                            name: "configId",
                                            xtype: "hidden"
                                        },
                                        {
                                            fieldLabel: "Type",
                                            ref: "../configType",
                                            name: "configType",
                                            readOnly: true
                                        },
                                        {
                                            fieldLabel: "Name",
                                            ref: "../configName",
                                            name: "configName",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "Project Id",
                                            ref: "../generalProjectId",
                                            name: "generalProjectId",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "User Identifier",
                                            ref: "../generalUserId",
                                            name: "generalUserId",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "User Secret",
                                            ref: "../generalUserSecret",
                                            name: "generalUserSecret",
                                            allowBlank: false
                                        },
                                        {
                                            fieldLabel: "Callback URL",
                                            ref: "../callbackUrl",
                                            name: "callbackUrl",
                                            vtype: 'url'
                                        },
                                        {
                                            fieldLabel: "Retrieval Type",
                                            ref: "../retrievalType",
                                            name: "retrievalType",
                                            value: "published",
                                            allowBlank: "false",
                                            type: "select",
                                            xtype: "selection",
                                            options: "/services/translation/retrievaltypes",
                                            optionsRoot: "retrievalTypes",
                                            optionsTextField: "label",
                                            optionsValueField: "value"
                                        }
                                    ]
                                }
                            ]
                        },
                        this.getTranslationConfigsPanel()
                    ]
                }
            ]
        });

        CQ.Smartling.DictionaryMappingDialog.superclass.constructor.call(this, config);
    },

    isValid: function (form) {
        var dialog = this;
        var valid = true;
        form.items.each(function (f) {
            if (f.isCheckbox && !f.el) {
                return;
            }

            if (f.getName() == "dictionryTargetInfo" && f.getValue().length < 1) {
                dialog.invalidMessage = dialog.invalidMessage + "Error for field " + f.getName() + ": 'You must add at least one item' <br/>";
                valid = false;
            }

            if (!f.validate()) {
                if (f.getErrors) {
                    var errors = f.getErrors(f.getValue());
                    var startIndex = errors.length > 1 ? 1 : 0;
                    for (i = startIndex; i < errors.length; i++) {
                        dialog.invalidMessage = dialog.invalidMessage + "Error for field " + f.getName() + ": '" + errors[i] + "' <br/>";
                    }
                } else {
                    dialog.invalidMessage = dialog.invalidMessage + "Error for field " + f.getName() + ": '" + f.blankText + "' <br/>";
                }
                valid = false;
            }
        });

        valid = this._validateCustomerKeys(valid);
        valid = this._validateTargetFields(valid);

        return valid;
    },

    _validateCustomerKeys: function _checkProxySettings(valid) {
        var generalSettings = this.items.items[0].items.items[0].items.items[0];
        var generalProjectId = generalSettings.generalProjectId.getValue();
        var RegExp = /^\w{9}$/;
        var errorMessage = "Project Id wrong configured. It should look like 'XXXXXXXXX'. <br/>";
        valid = this._mutchRegexp(valid, generalProjectId, RegExp, errorMessage);

        var generalUserSecret = generalSettings.generalUserSecret.getValue();
        RegExp = /.{54,55}/;
        errorMessage = "User Secret key wrong configured. It should look like 'X{54,55}'. <br/>";
        valid = this._mutchRegexp(valid, generalUserSecret, RegExp, errorMessage);

        var generalUserId = generalSettings.generalUserId.getValue();
        RegExp = /.{30}/;
        errorMessage = "User Identifier key wrong configured. It should look like 'X{30}'. <br/>";
        valid = this._mutchRegexp(valid, generalUserId, RegExp, errorMessage);

        return valid;
    },

    _mutchRegexp: function _mutchRegexp(isValid ,variable, regexp, errorMessage) {
        if (!regexp.test(variable)) {
            isValid = false;
            this.invalidMessage += errorMessage;
        }
        return isValid;
    },

    getTargets : function() {
        var panelItems = this.items.items[0].items.items[0];
        return panelItems.items.items[1].dictionaryTargetInfoPanel.items.items[0].getValue();
    },

    save: function () {
        this.invalidMessage = "Next fields are filled incorrectly: <br/>";
        var form = this.form.getForm();
        if (!this.isValid(form)) {
            CQ.Ext.Msg.show({
                title: "Validation Error",
                msg: this.invalidMessage,
                buttons: CQ.Ext.MessageBox.OK,
                icon: CQ.Ext.MessageBox.ERROR
            });
            return;
        }

        var isNewConfig = !this.strorageRecord;
        var strorageRecord = isNewConfig ? new this.configStore.recordType({ key: '' }) : this.strorageRecord;
        var panelItems = this.items.items[0].items.items[0];

        strorageRecord.beginEdit();
        strorageRecord.set("configType", panelItems.items.items[0].configType.getValue());
        strorageRecord.set("configName", panelItems.items.items[0].configName.getValue());
        strorageRecord.set("generalProjectId", panelItems.items.items[0].generalProjectId.getValue());
        strorageRecord.set("generalUserSecret", panelItems.items.items[0].generalUserSecret.getValue());
        strorageRecord.set("generalUserId", panelItems.items.items[0].generalUserId.getValue());
        strorageRecord.set("callbackUrl", panelItems.items.items[0].callbackUrl.getValue());
        strorageRecord.set("retrievalType", panelItems.items.items[0].retrievalType.getValue());
        strorageRecord.set("sourcePath", panelItems.items.items[1].sourcePath.getValue());
        strorageRecord.set("sourceLocalePath", panelItems.items.items[1].sourceLocalePath.getValue());

        var targetInfoValues = panelItems.items.items[1].dictionaryTargetInfoPanel.items.items[0].getValue();
        var targetInfos = [];
        for (i = 0; i < targetInfoValues.length; i++) {
            targetInfos[i] = JSON.parse(targetInfoValues[i]);
        }
        strorageRecord.set("targetInfos", JSON.stringify(targetInfos));

        strorageRecord.endEdit();

        if (isNewConfig) {
            this.configStore.insert(0, strorageRecord);
        }
        this.configStore.saveAll();

        this.hide();
    },

    addTranslations: function () {
        this.strorageRecord = null;

        var tabs = this.form.items.items[0].items;

        for (i = 0; i < tabs.length; i++) {
            this.initTab(i, this);
            tabs.items[i].items.items[0].items.each(function (f) {
                if (f.name != "dictionaryTargetInfoPanel") {
                    f.setValue("");
                }
            }, this);
        }

        var panelItems = this.items.items[0].items.items[0];
        panelItems.items.items[0].configType.setValue(this.getConfigType());
        panelItems.items.items[0].retrievalType.setValue("published");

        panelItems.items.items[1].dictionaryTargetInfoPanel.items.items[0].setValue("");

        this.form.items.items[0].setActiveTab(0);
        this.show();
    },

    getConfigType: function() {
        return "DICTIONARY";
    },

    initTab: function (i, dialog) {
        dialog.form.items.items[0].setActiveTab(i);
        if (!dialog.rendered) {
            dialog.render(CQ.Util.getRoot());
        }
        dialog.beforeShow();
    },

    editTranslations: function (strorageRecord) {
        this.strorageRecord = strorageRecord;
        this._fillDialogFromRecord(strorageRecord);

    },

    cloneTranslations: function (strorageRecord) {
        this.strorageRecord = null;
        this._fillDialogFromRecord(strorageRecord);
    },

    _fillDialogFromRecord: function _fillDialogFromRecord(strorageRecord) {
        var tabs = this.form.items.items[0].items;

        for (i = 0; i < tabs.length; i++) {
            this.initTab(i, this);
            tabs.items[i].items.items[0].items.each(function (f) {
                if (this.strorageRecord == null && typeof f.name != "undefined" && f.name == "configName") {
                    f.setValue(strorageRecord.get(f.name) + " Copy");
                } if (typeof f.name != "undefined" && f.name != "dictionaryTargetInfoPanel") {
                    f.setValue(strorageRecord.get(f.name));
                }
            }, this);
        }

        var panelItems = this.items.items[0].items.items[0];
        var targetInfos = JSON.parse(strorageRecord.get("targetInfos"));
        for (i = 0; i < targetInfos.length; i++) {
            targetInfos[i] = JSON.stringify(targetInfos[i]);
        }
        panelItems.items.items[1].dictionaryTargetInfoPanel.items.items[0].setValue(targetInfos);

        this.form.items.items[0].setActiveTab(0);

        this.show();
    },

    getTargetInfos: function (panelItems) {
        return panelItems.items.items[1].dictionaryTargetInfoPanel.items.items[0].getValue();
    }
});
CQ.Ext.reg("smartlingDictionaryMappingdialog", CQ.Smartling.DictionaryMappingDialog);