CQ.Ext.ns('CQ.SC.Components.LogViewer');

/**
 * @class CQ.SC.Components.LogViewer
 * The exporter enables the user to export Pcs system.
 * @constructor
 * Creates a new exporter.
 * @param {Object} config The config object
 * CQ.Ext.Viewport
 */
CQ.SC.Components.LogViewer = CQ.Ext.extend(CQ.Ext.Viewport, {
    mainConfig: null,
    constructor: function(config) {
        /*
         * Adding custom CSS class that allow override CQ styles
         */
        CQ.Ext.get('CQ').addClass('log-viewer');

        this.mainConfig = config;
        this.results = document.createElement('iframe');
        this.results.id = 'results-log-viewer';
        this.results.name = 'resultsLogViewer';
        this.results.height = '100%';
        this.results.width = '100%';
        this.results.onload = this.onResultsLoad;
        this.results.onreadystatechange = this.onResultsLoad;

        var exporter = this;

        CQ.SC.Components.LogViewer.superclass.constructor.call(this, {
            id: 'cq-logviewer',
            layout: 'border',
            renderTo: CQ.Util.getRoot(),
            items: [ {
                id: 'cq-logviewer-wrapper',
                xtype: 'panel',
                region: 'center',
                layout: 'border',
                border: false,
                items: [
                        {
                            id: 'cq-header',
                            xtype: 'container',
                            autoEl: 'div',
                            region: 'north',
                            items: [ {
                                xtype: 'panel',
                                border: false,
                                layout: 'column',
                                cls: 'cq-header-toolbar',
                                items: [ new CQ.Switcher({}),
                                        new CQ.UserInfo({}),
                                        new CQ.HomeLink({}) ]
                            } ]
                        },
                        {
                            xtype: 'form',
                            id: 'log-viewer-form',
                            title: 'Log Viewer',
                            region: 'center',
                            fileUpload: true,
                            standardSubmit: true,
                            autoScroll: true,
                            border: false,
                            margins: '5 5 5 5',
                            autoHeight: true,

                            items: [{
                                xtype: 'numberfield',
                                fieldLabel: 'Lines count:',
                                fieldDescription : 'Select last X lines from log, empty for all.',
                                allowNegative: false,
                                width: 60,
                                anchor: '40%',
                                name: 'linesCount',
                                allowBlank: true,
                                value: "50"
                                },
                                {
                                    xtype: 'hidden',
                                    name: 'download'
                                }],
                            buttonAlign: 'left',
                            buttons: [ {
                                id: 'log-viewer-btn-update',
                                text: 'Refresh',
                                scope: this,
                                handler: this.onButtonUpdateClick
                            },
                                {
                                id: 'log-viewer-btn-download',
                                text: 'Download',
                                tooltip : 'Download smartling logs and submissions as archive',
                                scope: this,
                                handler: this.onButtonDownloadClick
                            } ]
                        },
                        {
                            xtype: 'panel',
                            id: 'log-viewer-log',
                            region: 'south',
                            title: 'Smartling Log',
                            margins: '-5 5 5 5',
                            height: 600,
                            split: true,
                            collapsible: true,
                            collapsed: false,
                            items: [ new CQ.Ext.BoxComponent({
                                autoEl: {
                                    tag: 'div'
                                },
                                style: {
                                    width: '100%',
                                    height: '100%',
                                    margin: '-2px'
                                },
                                listeners: {
                                    render: function(wrapper) {
                                        new CQ.Ext.Element(exporter.results)
                                            .appendTo(wrapper.getEl());
                                    }
                                }
                            }) ]
                        } ]
            } ]
        });
    },
    onButtonUpdateClick: function(button) {
        var form = CQ.Ext.getCmp('log-viewer-form').getForm();
        form.items.items[1].setValue(false);
        var config = button.scope.mainConfig;

        if (form.isValid()) {
            button.setDisabled(true);

            var log = CQ.Ext.getCmp('log-viewer-log');
            log.expand();

            CQ.Ext.get('log-viewer-form').mask();
            // submit form
            form.getEl().dom.action = CQ.HTTP.externalize(config.url);
            form.getEl().dom.method = 'GET';
            form.getEl().dom.target = 'resultsLogViewer';
            form.submit();
        }
    },

    onButtonDownloadClick: function(button) {
        var form = CQ.Ext.getCmp('log-viewer-form').getForm();
        form.items.items[1].setValue(true);

        var config = button.scope.mainConfig;

        if (form.isValid()) {
            // submit form
            form.getEl().dom.action = CQ.HTTP.externalize(config.url);
            form.getEl().dom.method = 'GET';
            form.getEl().dom.target = 'resultsLogViewer';
            form.submit();
        }
    },

    onResultsLoad: function() {
        CQ.Ext.getCmp('log-viewer-btn-update').setDisabled(false);
        CQ.Ext.get('log-viewer-form').unmask();
    }
});

CQ.Ext.reg('smartling-log-viewer', CQ.SC.Components.LogViewer);
