CQ.Ext.ns("CQ.Smartling");

CQ.Ext.onReady(function () {

    var i18nPath = null;

    var fields = ['isAssetProperty', 'resourceType', 'properties'];

    var configStore = new CQ.Ext.data.JsonStore({
        root: 'records',
        idProperty: 'resourceType',

        url: '/services/translation/propertymapping',

        autoSave: false,
        writer: new CQ.Ext.data.JsonWriter({
            encode: false,
            listful: true,
            writeAllFields: true
        }),
        isDirty: function () {
            if (this.getModifiedRecords().length > 0) {
                return true;
            }
            var dirty = false;
            this.each(function (record) {
                if (record.deleted) {
                    dirty = true;
                    return false;
                }
            });
            return dirty;
        },
        markForRemove: function (record) {
            if (record) {
                record.deleted = true;
            }
        },
        proxy: new CQ.Ext.data.HttpProxy({
            api: {
                read: { url: '/services/translation/propertymapping', method: 'GET' },
                update: '/services/translation/propertymapping',
                create: '/services/translation/propertymapping',
                destroy: '/services/translation/propertymapping'
            },
            headers: {"Content-Type": "application/json; charset=utf-8"}
        }),

        saveAll: function () {
            // delete records marked for deletion
            this.each(function (record) {
                if (record.deleted) {
                    this.remove(record);
                }
            }, this);
            this.save();
        },
        // overwrite to handle simple response with only "success: true" instead of
        // returning all ids of the written records (replacing DataReader.realize)
        onCreateRecords: function (success, rs, data) {
            this.commitChanges();
            this.each(function (record) {
                record.phantom = false;
            }, this);
        },

        fields: fields
    });
    //store.setDefaultSort('string', 'desc');


    var propertyGrid;

    configStore.on("beforesave", function () {
        propertyGrid.body.mask("Saving...");
    });
    configStore.on("save", function () {
        propertyGrid.body.unmask();
    });

    function loadPath() {
        configStore.reload();
    }


    var editPropertyDialog = new CQ.Smartling.PropertyMappingDialog({
        languages: "de",
        configStore: configStore
    });
    //      var fields = ['propertyMappingId','isAssetProperty','resourceType','propertyName'];
    var cmCfg = {
        columns: [
            {
                header: "Type",
                dataIndex: 'isAssetProperty',
                width: 50
            },
            {
                id: 'resourceType',
                header: "Sling Resource Type",
                dataIndex: 'resourceType',
                width: 200
            },
            {
                header: "Property Names",
                dataIndex: 'properties',
                flex: 1,
                id: 'properties'
            }
        ]
    };

    function showMenu(grid, index, event) {
        event.stopEvent();
        var record = grid.getStore().getAt(index);
        var menu = new CQ.Ext.menu.Menu({
            items: [
                {
                    text: 'Edit',
                    handler: function () {
                        editPropertyDialog.editTranslations(record);
                    }
                },
                {
                    text: 'Delete',
                    handler: function () {
                        grid.getStore().markForRemove(record);
                        grid.getStore().saveAll();
                    }
                }
            ]
        }).showAt(event.xy);
    }

    propertyGrid = new CQ.Ext.grid.EditorGridPanel({
        region: "center",
        //title: "Strings",
        title: 'Property Mapping',
        layout: 'fit',
        margins: '5 5 5 5',
        autoExpandColumn: 'properties',
        selModel: new CQ.Ext.grid.RowSelectionModel(),
        //trackMouseOver:false,
        //disableSelection:true,
        loadMask: true,
        store: configStore,
        cm: new CQ.Ext.grid.ColumnModel(cmCfg),
        listeners: {
            'rowcontextmenu': function (grid, index, event) {
                showMenu(grid, index, event);
            }
        }
    });
    propertyGrid.getSelectionModel().on("selectionchange", function (sm) {
    });


    var propertyViewport = new CQ.Ext.Viewport({
        layout: "border",
        border: true,
        defaults: {
            border: true,
            margins: '5 5 5 5'
        },
        items: [
            {
                region: "center",
                layout: "fit",
                items: {
                    layout: "border",
                    title: "Smartling: Property Mapping",
                    border: false,
                    tbar: [
                        {
                            text: "Save",
                            id: "strings-save-button",
                            iconCls: "save-icon",
                            disabled: false,
                            handler: function () {
                                configStore.saveAll();
                            }
                        },
                        {
                            text: "Add",
                            iconCls: "add-icon",
                            handler: function () {
                                propertyGrid.stopEditing();
                                editPropertyDialog.addTranslations();
                            }
                        },
                        {
                            text: "Reset & Refresh",
                            iconCls: "refresh-icon",
                            handler: function () {
                                configStore.reload();
                            }
                        }
                    ],
                    items: [
                        propertyGrid
                    ]
                }
            }
        ]
    });

    loadPath();

});