CQ.Ext.ns('SMARTLING.ui');

SMARTLING.ui.common = SMARTLING.ui.common || {};

function constructCancelSubmissionObjects(selections) {
    var cancelSubmissions = [];
    for (var i = 0; i < selections.length; i++) {
        cancelSubmissions.push({
            fileUri: selections[i].data.fileUri,
            sourcePath: selections[i].data.sourcePath,
            locale: selections[i].data.locale
        });
    }
    return cancelSubmissions;
}

function canceledAmongSelected(selections) {
    for (var i = 0; i < selections.length; i++) {
        if (selections[i].data.state === 'Canceled') {
            return true;
        }
    }
    return false;
}

function allCanceled(selections) {
    for (var i = 0; i < selections.length; i++) {
        if (selections[i].data.state != 'Canceled') {
            return false;
        }
    }
    return true;
}

function getCancelMessage(grid) {
    return "Translation for selected items and locales will be canceled."
        + (canceledAmongSelected(grid.selModel.getSelections()) ?
            ' Not all items will be canceled because they have improper state.' : '');
}

function getNodePaths(selections, addAllPaths) {
    var selected = {submissions: [], nodes: []};
    for (var i = 0; i < selections.length; i++) {
        if (addAllPaths || selections[i].data.state != 'Canceled') {
            if (selections[i].data.nodePath) {
                selected.submissions.push(selections[i].data.nodePath);
            } else {

            }
            selected.nodes.push(selections[i].data.sourcePath);
        }
    }
    return selected;
}

SMARTLING.ui.common.TranslationAction = CQ.Ext.extend(CQ.Ext.Action, {

    constructor: function (config, grid) {
        var action = this;
        switch (config.action) {
            case 'download':
                config.handler = function () {
                    action.downloadTranslation(grid);
                };
                break;
            case 'cancel':
                config.handler = function () {
                    action.cancelTranslations(grid);
                };
                break;
            case 'preview':
                config.handler = function () {
                    action.previewTranslation(grid);
                };
                break;
        }

        SMARTLING.ui.common.TranslationAction.superclass.constructor.call(action, config);
    },

    previewTranslation: function (grid) {
        var selections = grid.selModel.getSelections();
        if (selections.length === 0) {
            CQ.Ext.Msg.show({
                title: 'Preview not available',
                msg: 'Select page for preview',
                width: 300,
                buttons: CQ.Ext.Msg.OK,
                multiline: false
            });
        } else {
            for (var i = 0; i < selections.length; i++) {
                var data = selections[i].data;
                if (data.configType === 'CONTENT' && data.state && data.targetPath !== null) {
                    window.open('/apps/smartling/preview.html?source=' + data.sourcePath + '&target=' + data.targetPath);

                } else {
                    CQ.Ext.Msg.show({
                        title: 'Warning',
                        msg: 'Preview mode is not available',
                        icon: CQ.Ext.Msg.INFO,
                        buttons: CQ.Ext.Msg.OK
                    });
                }
            }
        }
    },

    downloadTranslation: function (grid) {
        var selections = grid.selModel.getSelections();

        if (allCanceled(selections)) {
            CQ.Ext.Msg.show({
                title: 'Download submissions',
                msg: 'Submissions can not be downloaded, they canceled',
                width: 300,
                buttons: CQ.Ext.Msg.OK,
                multiline: false
            });

            return;
        }
        CQ.Ext.Msg.confirm("Confirmation", "Are you sure you want to proceed?" + (canceledAmongSelected(selections) ?
                ' Not all items will be downloaded because they have improper state.' : ''), function (btn) {
            if (btn === 'yes') {
                var nodePaths = getNodePaths(selections, false).submissions;
                grid._mask();
                CQ.Ext.Ajax.request({
                    method: 'POST',
                    url: '/services/smartling/submission/bulk/download',
                    params: {
                        nodePaths: nodePaths
                    },
                    callback: function (options, success, response) {
                        grid._unmask();
                        CQ.Ext.Msg.show({
                            title: 'Download',
                            msg: success ? 'Successfully downloaded.' : 'Failed to download translations. ' + CQ.Ext.decode(response.responseText).errors,
                            width: 300,
                            buttons: CQ.Ext.Msg.OK,
                            multiline: false
                        });
                        grid.reloadGrid(null);
                    }
                });
            }
        });

    },

    cancelTranslations: function (grid) {
        if (allCanceled(grid.selModel.getSelections())) {
            CQ.Ext.Msg.show({
                title: 'Cancel submissions',
                msg: 'Submission is already canceled',
                width: 300,
                buttons: CQ.Ext.Msg.OK,
                multiline: false
            });

            return;
        }

        CQ.Ext.Msg.confirm("Confirmation", getCancelMessage(grid), function (btn) {
            if (btn === 'yes') {
                var cancelSubmissionsDto = constructCancelSubmissionObjects(grid.selModel.getSelections());
                grid._mask();
                CQ.Ext.Ajax.request({
                    method: 'POST',
                    url: '/services/smartling/submission/bulk/cancel',
                    params: {
                        cancelSubmissions: CQ.Ext.encode(cancelSubmissionsDto)
                    },
                    callback: function (options, success, response) {
                        grid._unmask();
                        CQ.Ext.Msg.show({
                            title: 'Cancel submissions',
                            msg: success ? 'Selected submissions have been cancelled.' : 'Failed to cancel selected submissions. ' + CQ.Ext.decode(response.responseText).errors,
                            width: 300,
                            buttons: CQ.Ext.Msg.OK,
                            multiline: false
                        });
                        grid.reloadGrid(null);
                    }
                });
            }
        });
    }

});
