CQ.Ext.ns('SMARTLING.ui.dialog');

SMARTLING.ui.dialog.TargetLocalesPanel = CQ.Ext.extend(CQ.Ext.form.FieldSet, {
    localesName: null,
    getSelectedLocales: function () {
        return this.getSelectedLocalesArray();
    },

    clearSelectedLocales: function () {
        $('[name="' + this.localesName + '"]').each(function (index, el) {
            $(el).prop('checked', false);
        });
    },

    constructor: function (config) {
        config = config || {};

        var defaults = {
            localesName: 'locales'
        };

        config = CQ.Util.applyDefaults(config, defaults);
        this.localesName = config.localesName;

        var originalLanguageOptions;
        var selectedLocales = [];

        this.getSelectedLocalesArray = function(){
            return selectedLocales;
        };

        function localesCheckboxGroupListener(selectedLocales) {
            $('[name="' + config.localesName + '"]').each(function (index, el) {
                $(el).change(
                    function () {
                        if ($(this).is(':checked')) {
                            if ($.inArray(this.value, selectedLocales) == -1) {
                                selectedLocales.push(this.value);
                            }
                        } else {
                            var index = $.inArray(this.value, selectedLocales);
                            if (index > -1) {
                                selectedLocales.splice(index, 1);
                            }
                        }
                    });
            });
            $('[name="' + config.localesName + '"]').filter(function (index, el) {
                return $.inArray(el.value, selectedLocales) != -1;
            }).each(function (index, el) {
                $(el).prop('checked', true);
            });
        }

        function jobLocalesCheckboxGroup() {
            return new CQ.form.Selection({
                name: config.localesName,
                type: 'checkbox',
                allowBlank: false,
                autoScroll: true,
                height: 100,
                blankText: 'At least one languages should be selected',
                options: '/services/smartling/locales?path=' + config.paths[0],
                optionsRoot: '.',
                optionsTextField: 'name',
                optionsValueField: 'code',
                listeners: {
                    added: function () {
                        originalLanguageOptions = selectedLocaleGroup.optionItems.items;

                    },
                    afterlayout: function () {
                        localesCheckboxGroupListener(selectedLocales);
                    }
                },
                tpl: [
                    '<tpl for=".">',
                    '    <div class="x-form-item  x-hide-label" tabindex="-1" style="overflow: hidden;">',
                    '        <label style="width:100px;" class="x-form-item-label">&nbsp;</label>',
                    '        <div class="x-form-element" style="padding-left:105px">',
                    '            <div class="x-form-check-wrap" style="width: 211px;">',
                    '                <input type="checkbox" autocomplete="off" name="' + config.localesName + '" class=" x-form-checkbox x-form-field" value="{inputValue}">',
                    '                <label class="x-form-cb-label-hidden">{boxLabel}</label>',
                    '            </div>',
                    '        </div>',
                    '        <div class="x-form-clear-left"/>',
                    '    </div>',
                    '</tpl>'
                ],
                tplWriteMode: 'overwrite'
            });
        }

        var selectedLocaleGroup = jobLocalesCheckboxGroup();

        CQ.Ext.applyIf(config, {
            fieldLabel: 'Target Locales *',
            headerAsText: false,
            autoHeight: true,
            layout: 'anchor',
            hideBorders: true,
            items: [
                {
                    xtype: 'fieldset',
                    headerAsText: false,
                    autoHeight: true,
                    layout: 'column',
                    anchor: '100%',
                    style: 'margin-bottom: 0px;padding-bottom: 5px;padding-top: 0px;padding-left: 1px',
                    hideBorders: true,
                    items: [
                        {
                            xtype: 'checkbox',
                            name: 'select-all-locales',
                            hideLabel: true,
                            listeners: {
                                check: function (element) {
                                    $('[name="' + config.localesName + '"]').each(function () {

                                        $(this).prop("checked", element.checked).trigger("change");

                                    });
                                }
                            }
                        },
                        {
                            xtype: 'textfield',
                            name: 'locales-search',
                            enableKeyEvents: true,
                            anchor: '100%',
                            style: 'width: calc(100% - 27px)',
                            listeners: {
                                keyup: function (event) {
                                    var elements = originalLanguageOptions.slice();
                                    if (event.getValue()) {
                                        elements = originalLanguageOptions.slice().filter(function (value) {
                                            return value.boxLabel.toLowerCase().startsWith(event.getValue().toLowerCase()) ||
                                                value.inputValue.toLowerCase().startsWith(event.getValue().toLowerCase());
                                        })
                                    }
                                    var sortedLocales = elements.sort(function (a, b) {
                                        var labelA = a.boxLabel;
                                        var labelB = b.boxLabel;
                                        if (labelA < labelB) {
                                            return -1;
                                        }
                                        if (labelA > labelB) {
                                            return 1;
                                        }
                                        return 0;
                                    });
                                    selectedLocaleGroup.update(sortedLocales);
                                    localesCheckboxGroupListener(selectedLocales);
                                }
                            }
                        }]
                },
                selectedLocaleGroup
            ]
        });
        SMARTLING.ui.dialog.TargetLocalesPanel.superclass.constructor.call(this, config);
    }
});

SMARTLING.ui.dialog.SendForTranslationDialog = CQ.Ext.extend(CQ.Ext.Window, {
    constructor: function (config) {
        function displayServerError(response, defaultMessage) {
            var messageTemplate = new CQ.Ext.XTemplate(
                '{message}' +
                '<tpl if="typeof(details) !== \'undefined\' && this.isNotEmpty(details)">' +
                '   <br/><br/>' +
                '   Details:<br/>' +
                '   <ul style="border: 2px inset; margin-top: 5px; padding-left: 5px; min-height: 2em">' +
                '   <tpl for="details">' +
                '       <li> - {.}</li>' +
                '   </tpl>' +
                '   </ul>' +
                '</tpl>',
                {
                    isNotEmpty: function (array) {
                        return array !== null && array.length > 0;
                    }
                }
            );

            var responseData;
            try {
                responseData = CQ.Ext.decode(response.responseText);
            } catch (e) {
                responseData = {};
            }

            var errorInfo = CQ.Util.applyDefaults(responseData.error, {
                message: defaultMessage
            });

            CQ.Ext.Msg.show({
                title: 'Error',
                msg: messageTemplate.apply(errorInfo),
                width: 500,
                icon: CQ.Ext.Msg.ERROR,
                buttons: CQ.Ext.Msg.OK
            });
        }


        function loadingShow() {
            var zIndex = Math.max.apply(null, $('.x-shadow').map(function (index, el) {
                    return $(el).css('z-index');
                }).get()) + 1;
            var loading = $('<div/>', {
                id: 'loading',
                style: 'z-index:' + zIndex
            }).appendTo(document.body);
            $('<img/>', {
                id: 'loading-image',
                src: '/apps/smartling/ui/widgets/dialog/icons/loading.gif'
            }).appendTo(loading);
        }

        function loadingHide() {
            $('#loading').remove();
        }

        var loadJobDetails = function (fieldSet, value) {
            var descriptionField = fieldSet.findByType('textarea', true)[0];
            var contentField = fieldSet.findByType('panel', true)[0];

            function groupBy(arr, keyGetter, valGetter) {
                var map = {};
                arr.forEach(function (item) {
                    var key = keyGetter(item);
                    var collection = map[key];
                    if (!collection) {
                        map[key] = [valGetter(item)];
                    } else {
                        collection.push(valGetter(item));
                    }
                });
                return map;
            }

            function toSourcePathTargetLocalesArray(map) {
                var result = [];
                for (var property in map) {
                    if (map.hasOwnProperty(property)) {
                        result.push({sourcePath: property, targetLocales: map[property]});
                    }
                }
                return result;
            }

            CQ.Ext.Ajax.request({
                    method: 'GET',
                    url: '/services/smartling/classic/jobs/details',
                    params: {
                        path: config.paths[0],
                        uid: value.data.uid
                    },
                    success: function (response) {
                        var jobDetails = CQ.Ext.util.JSON.decode(response.responseText);
                        descriptionField.setValue(jobDetails.description);
                        var content = jobDetails.content;
                        var properties = content.map(function (el) {
                            return el.properties;
                        });
                        var groupped = toSourcePathTargetLocalesArray(groupBy(properties, function (el) {
                                return el.sourcePath;
                            }, function (el) {
                                return el.target_locale;
                            }
                        )).sort(function (a, b) {
                            var sourceA = a.sourcePath;
                            var sourceB = b.sourcePath;
                            if (sourceA < sourceB) {
                                return -1;
                            }
                            if (sourceA > sourceB) {
                                return 1;
                            }
                            return 0;
                        });
                        contentField.update(groupped);
                    }
                }
            );
        };

        var submitForTranslation = function (dialog, params, callback) {
            CQ.Ext.Ajax.request({
                method: 'POST',
                url: '/services/smartling/classic/jobs',
                params: {
                    paths: params.paths,
                    excludedPaths: params.excludedPaths,
                    authorize: params.authorize,
                    bulk: params.bulk,
                    targetLocaleCodes: params.targetLocaleCodes,
                    jobUid: params.jobUid,
                    jobName: params.jobName,
                    jobDescription: params.jobDescription,
                    jobDueDate: params.jobDueDate
                },
                success: function () {
                    try {
                        dialog.close();
                    } catch (err) {
                    }
                    loadingHide();
                    CQ.Ext.Msg.show({
                        title: 'Success',
                        msg: 'Content was sent for translation',
                        icon: CQ.Ext.Msg.INFO,
                        width: 500,
                        buttons: CQ.Ext.Msg.OK
                    });
                    callback();
                },
                failure: function (response) {
                    loadingHide();
                    displayServerError(response, 'Failed to request translation');
                }
            });
        };

        var bulkTranslationDialog = null;

        function openBulkTranslationDialog(callback) {
            if (bulkTranslationDialog === null) {
                bulkTranslationDialog = new SMARTLING.ui.sidekick.BulkTranslateDialog({
                    'rootName': document.title,
                    'rootPath': CQ.WCM.getPagePath(),
                    'callback': callback
                });
            }
            bulkTranslationDialog.show();
        }

        function jobDueDate() {
            return new CQ.form.DateTime({
                name: 'dueDate',
                fieldLabel: 'Due Date',
                dateConfig: {
                    minValue: function () {
                        var tomorrow = new Date();
                        tomorrow.setDate(tomorrow.getDate() + 1);
                        return tomorrow;
                    }()
                }
            });
        }

        var createJobDueDate = jobDueDate();
        var addToJobDueDate = jobDueDate();

        function jobDescription() {
            return new CQ.Ext.form.TextArea({
                name: 'jobDescription',
                fieldLabel: 'Description',
                maxLength: 255
            });
        }

        var createJobDescription = jobDescription();
        var addToJobDescription = jobDescription();

        var jobName = new CQ.Ext.form.TextField({
            name: 'jobName',
            fieldLabel: 'Name',
            allowBlank: false,
            maxLength: 120
        });

        function authorizeJob() {
            return new CQ.Ext.form.Checkbox({
                name: 'authorize',
                fieldLabel: 'Authorize job'
            });
        }

        var createJobAuthorize = authorizeJob();
        var addToJobAuthorize = authorizeJob();

        function includeSubpages() {
            return new CQ.form.Selection({
                type: 'checkbox',
                name: 'bulkModeEnabled',
                inputValue: true,
                defaultValue: false,
                cls: 'bulk-submit-checkbox',
                fieldLabel: 'Bulk Submit',
                boxLabel: '(<a href="#">configure</a>)',
                listeners: {
                    afterrender: function (checkbox) {
                        checkbox.getEl().on('click', function (e) {
                            e.preventDefault();
                            openBulkTranslationDialog(function () {
                                checkbox.setValue(true);
                            });
                        }, null, {delegate: 'a'});
                    }
                }
            });
        }

        var createPageIncludeSubpages = includeSubpages();
        var addToJobIncludeSubpages = includeSubpages();

        var jobContent = new CQ.Ext.Panel({
            fieldLabel: 'Content',
            cls: 'job-content-list',
            height: 100,
            autoScroll: true,
            tpl: [
                '<tpl for=".">',
                '{sourcePath} ({targetLocales})<br>',
                '</tpl>'
            ]
        });

        var createJobTargetLocales = new SMARTLING.ui.dialog.TargetLocalesPanel({
            localesName: 'create-job-locales',
            paths: config.paths
        });
        var addToJobTargetLocales = new SMARTLING.ui.dialog.TargetLocalesPanel({
            localesName: 'add-to-job-locales',
            paths: config.paths
        });

        var addToJobUid;
        var addToJobName;

        var selectJobElement = new CQ.Ext.form.ComboBox({
            name: 'jobUid',
            fieldLabel: 'Job',
            allowBlank: false,
            store: new CQ.Ext.data.Store({
                proxy: new CQ.Ext.data.HttpProxy({
                    'autoLoad': true,
                    url: '/services/smartling/classic/jobs?path=' + config.paths[0],
                    method: 'GET'
                }),
                reader: new CQ.Ext.data.JsonReader({
                    fields: [
                        {name: 'uid', mapping: 'uid'},
                        {name: 'name', mapping: 'name'},
                        {name: 'state', mapping: 'state'},
                        {name: 'dueDate', mapping: 'dueDate'}
                    ]
                })
            }),
            valueField: 'uid',
            tpl: [
                '<tpl for=".">',
                '<tpl if="state === \'AWAITING_AUTHORIZATION\'">',
                '<div class="x-combo-list-item">{name} (Unauthorized)</div>',
                '</tpl>',
                '<tpl if="state === \'IN_PROGRESS\'">',
                '<div class="x-combo-list-item">{name} (In progress)</div>',
                '</tpl>',
                '<tpl if="state === \'COMPLETED\'">',
                '<div class="x-combo-list-item">{name} (Completed)</div>',
                '</tpl>',
                '</tpl>'
            ],
            displayField: 'name',
            triggerAction: 'all',
            listeners: {
                select: function (selection, value) {
                    var jobUid = value.data.uid;
                    if (selection.previouseValue !== jobUid) {
                        selection.previouseValue = jobUid;
                        addToJobUid = value.data.uid;
                        addToJobName = value.data.name;
                        var dueDateField = selection.ownerCt.findByType('datetime', true)[0];
                        dueDateField.setValue(value.data.dueDate);
                        loadJobDetails(selection.ownerCt, value);
                    }
                }
            }
        });

        CQ.Util.applyDefaults(config, {
            title: 'Send for Translation',
            closable: true,
            resizable: true,
            modal: true,
            layout: 'fit',
            y: 50,
            autoHeight: true,
            width: 400,
            minWidth: 350,
            items: [
                {
                    xtype: 'tabpanel',
                    activeTab: 0,
                    autoHeight: true,
                    items: [
                        {
                            xtype: 'fieldset',
                            title: 'Create New Job',
                            headerAsText: false,
                            autoHeight: true,
                            defaults: {
                                anchor: '100%'
                            },
                            items: [
                                jobName,
                                createJobDescription,
                                createJobDueDate,
                                createJobAuthorize,
                                createPageIncludeSubpages,
                                createJobTargetLocales,
                                {
                                    xtype: 'button',
                                    cls: 'job-button',
                                    text: 'Create job',
                                    scale: 'small',
                                    anchor: 'absolute',
                                    width: 91,
                                    scope: this,
                                    handler: function () {
                                        if (!jobName.getValue() || createJobTargetLocales.getSelectedLocales().length == 0) {
                                            CQ.Ext.Msg.show({
                                                title: 'Validation Error',
                                                msg: 'Name and locale should be selected',
                                                buttons: CQ.Ext.MessageBox.OK,
                                                icon: CQ.Ext.MessageBox.ERROR
                                            });
                                        } else {
                                            loadingShow();
                                            submitForTranslation(this, {
                                                paths: config.paths,
                                                bulk: createPageIncludeSubpages.getValue(),
                                                excludedPaths: bulkTranslationDialog ? bulkTranslationDialog.getExcludedPaths() : [],
                                                authorize: createJobAuthorize.getValue(),
                                                targetLocaleCodes: createJobTargetLocales.getSelectedLocales(),
                                                jobName: jobName.getValue(),
                                                jobDescription: createJobDescription.getValue(),
                                                jobDueDate: createJobDueDate.getValue()
                                            }, function(){createJobTargetLocales.clearSelectedLocales()});
                                        }
                                    }
                                }
                            ]
                        },
                        {
                            xtype: 'fieldset',
                            title: 'Add to Existing Job',
                            autoHeight: true,
                            headerAsText: false,
                            defaults: {
                                anchor: '100%'
                            },
                            items: [
                                selectJobElement,
                                addToJobDescription,
                                addToJobDueDate,
                                addToJobAuthorize,
                                addToJobIncludeSubpages,
                                addToJobTargetLocales,
                                jobContent,
                                {
                                    xtype: 'button',
                                    cls: 'job-button',
                                    text: 'Add to job',
                                    scale: 'small',
                                    scope: this,
                                    anchor: 'absolute',
                                    width: 91,
                                    handler: function () {
                                        if (!addToJobUid || addToJobTargetLocales.getSelectedLocales().length == 0) {
                                            CQ.Ext.Msg.show({
                                                title: 'Validation Error',
                                                msg: 'Job and locale should be selected',
                                                buttons: CQ.Ext.MessageBox.OK,
                                                icon: CQ.Ext.MessageBox.ERROR
                                            });
                                        } else {
                                            loadingShow();
                                            submitForTranslation(this, {
                                                paths: config.paths,
                                                bulk: addToJobIncludeSubpages.getValue(),
                                                excludedPaths: bulkTranslationDialog ? bulkTranslationDialog.getExcludedPaths() : [],
                                                authorize: addToJobAuthorize.getValue(),
                                                jobUid: addToJobUid,
                                                jobName: addToJobName,
                                                targetLocaleCodes: addToJobTargetLocales.getSelectedLocales(),
                                                jobDescription: addToJobDescription.getValue(),
                                                jobDueDate: addToJobDueDate.getValue()
                                            },function(){addToJobTargetLocales.clearSelectedLocales()});
                                        }
                                    }
                                }
                            ]
                        }
                    ]
                }
            ]
        });

        SMARTLING.ui.dialog.SendForTranslationDialog.superclass.constructor.call(this, config);
    }
});