CQ.Ext.ns('SMARTLING.ui.sidekick');
SMARTLING.ui.sidekick.BulkTranslateDialog = CQ.Ext.extend(CQ.Dialog, {

    treePanel: null,

    constructor: function (config) {
        var treeRootConfig = CQ.Util.applyDefaults(config.treeRoot, {
            "name": config.rootPath,
            "text": config.rootName,
            "checked": true,
            "iconCls": "page",
            "draggable": false,
            "singleClickExpand": true,
            "expanded": true
        });

        var treeLoaderConfig = CQ.Util.applyDefaults(config.treeLoader, {
            "dataUrl": "/", // Not used. Just has to be not empty
            "requestMethod": "GET",
            "baseParams": {
                "predicate": "siteadmin",
                "_charset_": "utf-8"
            },
            "baseAttrs": {
                "singleClickExpand": true
            },
            "listeners": {
                "beforeload": function (loader, node) {
                    // Substring to get rid of the extra slash at the beginning
                    this.dataUrl = node.getPath().substring(1) + ".ext.json"
                }
            }
        });

        this.treePanel = new CQ.Ext.tree.TreePanel({
            "lines": CQ.themes.BrowseDialog.TREE_LINES,
            "bodyBorder": CQ.themes.BrowseDialog.TREE_BORDER,
            "bodyStyle": CQ.themes.BrowseDialog.TREE_STYLE,
            "autoScroll": true,
            "containerScroll": true,
            "root": new CQ.Ext.tree.AsyncTreeNode(treeRootConfig),
            "loader": new CQ.Ext.tree.TreeLoader(treeLoaderConfig),
            "defaults": {
                "draggable": false
            },
            listeners: {
                'beforechildrenrendered': function (node) {
                    var checked = node.getUI().isChecked();
                    node.eachChild(function (n) {
                        n.attributes.checked = checked;
                        n.attributes.iconCls = 'page'
                    });
                },
                'checkchange': function (node, checked) {
                    var toggleCheck = function (n) {
                        n.getUI().toggleCheck(checked);
                    };

                    this.suspendEvents(false);
                    if (checked) {
                        node.bubble(toggleCheck);
                    }
                    node.cascade(toggleCheck);
                    this.resumeEvents();
                }
            }
        });

        CQ.Util.applyDefaults(config, {
            "title": "Bulk Translate",
            "closable": true,
            "width": CQ.themes.BrowseDialog.WIDTH,
            "height": CQ.themes.BrowseDialog.HEIGHT,
            "minWidth": CQ.themes.BrowseDialog.MIN_WIDTH,
            "minHeight": CQ.themes.BrowseDialog.MIN_HEIGHT,
            "resizable": CQ.themes.BrowseDialog.RESIZABLE,
            "resizeHandles": CQ.themes.BrowseDialog.RESIZE_HANDLES,
            "autoHeight": false,
            "autoWidth": false,
            "callback": CQ.Ext.emptyFn,
            "cls": "cq-browsedialog",
            "ok": function () {
                this.hide();
                this.callback();
            },
            "buttons": CQ.Dialog.OKCANCEL,
            "items": [this.treePanel]
        });
        SMARTLING.ui.sidekick.BulkTranslateDialog.superclass.constructor.call(this, config);
    },

    getExcludedPaths: function() {
        function collectUncheckedNodes(node, list) {
            if (node.attributes.checked) {
                node.eachChild(function (child) {
                    collectUncheckedNodes(child, list)
                });
            } else {
                list.push(node.getPath().substring(1));
            }

            return list;
        }

        return collectUncheckedNodes(this.treePanel.getRootNode(), []);
    }
});
