;(function (document, $, smartlingApi) {

    function closeDialog() {
        $('#classic-smartling-job-dialog').get(0).hide();
    }

    /**
     * @param foundationUi == $(window).adaptTo('foundation-ui')
     * @param xhr ajax response object
     * @param defaultText default error message text (used as a fallback, if xhr.error.message is not specified)
     */
    function notifyAjaxError(foundationUi, xhr, defaultText) {
        var responseJson = $.parseJSON(xhr.responseText);
        var errorMessage = responseJson.error.message || defaultText;

        if (responseJson.error.details && responseJson.error.details.length > 0) {
            errorMessage += "<br/>Details:<ul>";
            responseJson.error.details.forEach(function (item) {
                errorMessage += '<li>' + item + '</li>';
            });
            errorMessage += "</ul>";
        }

        foundationUi.notify('Error', errorMessage, 'error');

    }

    function initializeControls() {

        var foundationUi = $(window).adaptTo('foundation-ui');
        var $dialog = $('#classic-smartling-job-dialog');
        var $tabs = $dialog.find('#classic-smartling-job-dialog-tabs');

        //Recenter dialog pop-up during changing of the selected dialog tab
        $tabs.on('coral-tabview:change', function() {
            $dialog.get(0).center();
        });

        $dialog.find('coral-datepicker').each(function () {
            this.min = moment().startOf('day').add(1, 'days').toDate();
        });

        // Form submission event
        $dialog.find('form')
            .on('foundation-form-submitted', function (e, success, xhr) {
                if (success) {
                    closeDialog();
                    foundationUi.notify('Success', 'Page was added to a job', 'success');
                } else {
                    notifyAjaxError(foundationUi, xhr, 'Failed to request translation');
                }
            })
            .on('reset', function () {
                this.querySelector('coral-checkbox[name="authorize"]').checked = true;
                this.querySelector('coral-checkbox[name="bulk"]').checked = false;
                // TODO: reset tags
            });

        // Load existing jobs to Job Selector
        $dialog.find('#classic-smartling-job-select')
            .on('coral-select:showitems', function (e) {
            var select = this;

            if (select.items.length === 0) {
                e.preventDefault();

                $.ajax({
                    url: '/services/smartling/classic/jobs?path=' + smartlingApi.getContentPath()
                }).done(function (data) {
                    data.forEach(function (job) {
                        var displayState;
                        if (job.state === 'AWAITING_AUTHORIZATION') {
                            displayState = '(Unauthorized)';
                        } else if (job.state === 'IN_PROGRESS') {
                            displayState = '(In progress)';
                        } else if (job.state === 'COMPLETED') {
                            displayState = '(Completed)'
                        }
                        else {
                            displayState = '(' + job.state + ')'
                        }
                        select.items.add({value: job.uid, content: {innerHTML: job.name + ' ' + displayState}});
                    });
                }).always(function () {
                    select.loading = false;
                });
            }
        }).on('change', function (e) {
            var $form = $(this).closest('form');
            var $dueDateFiled = $form.find('coral-datepicker[name="jobDueDate"]');
            var $jobDescriptionField = $form.find('textarea[name="jobDescription"]');
            var $targetLocales = $form.find('.js-languages-autocomplete');
            var $contentContainer = $('.js-sm-add-dialog-content');

            foundationUi.wait($form);
            $.ajax({
                url: '/services/smartling/classic/jobs/details?path=' + smartlingApi.getContentPath() + '&uid=' + e.target.value
            }).done(function (data) {
                $dueDateFiled.val(data.dueDate);
                $jobDescriptionField.val(data.description || '');
                $targetLocales.trigger('languageSelector:updateLocales', {values: data.targetLocaleIds});
                $contentContainer.trigger('content:updateSubmittedContent', {values: data.content});
            }).fail(function (xhr) {
                notifyAjaxError(foundationUi, xhr, 'Failed to load Job data');
                $dueDateFiled.val('');
                $jobDescriptionField.val('');
                $targetLocales.trigger('languageSelector:updateLocales', {values: []});
            }).always(function () {
                foundationUi.clearWait();
            });
        });

        // Process Tab switch (clear validation, update submit button)
        $tabs.on('coral-panelstack:change', function () {
            var $submitButton = $dialog.find('#classic-smartling-job-dialog-submit');
            var $form = $(this.panelStack.selectedItem).find('form');

            var $unselectedTab = $dialog.find('coral-tab[aria-selected="false"]').get(0);
            if ($submitButton.length) {
                $submitButton.get(0).label.innerText = $unselectedTab.label.innerText;
            }
            $submitButton.attr('form', $form.attr('id'));

            resetValidationState($dialog);
        });

        // Reset dialog state when closed (user inputs, validation errors)
        $dialog.on('coral-overlay:close', '*', function (e) {
            e.stopPropagation();
        }).on('coral-overlay:close', function () {
            // reset inputs
            $(this).find('form').each(function () {
                this.reset();
            });

            resetValidationState($dialog);
            // pre-select first tab on the dialog
            $tabs.get(0).tabList.items.first().selected = true;

            $(document).trigger('progressBar:reload');
        });

        // show validation error, hidden by resetValidationState()
        $dialog.on("foundation-validation-invalid", function (event) {
            var target = $(event.target);
            target.parent().find(".coral-Form-fielderror").show();
        });
    }

    /**
     * @param $dialog root element to reset validation on
     */
    function resetValidationState($dialog) {
        // reset validation state
        $dialog.find(".coral-Form-fielderror").hide();
        $dialog.find(".is-invalid").removeAttr("invalid");
    }

    $(document).ready(initializeControls);

})(document, Granite.$, window.SmartlingApi = window.SmartlingApi || {});