;(function (document, $) {
    var SELECT_ALL_VALUE = 'select-all';

    $(document).ready(function initialize() {
        $('.js-language-selector-field').each(function () {
            var $selector = $(this);
            var languageSelectorField = CUI.Widget.fromElement('Autocomplete', $selector);

            // Add "All" option to languages dropdown
            languageSelectorField._selectListWidget.addOption({
                value: SELECT_ALL_VALUE,
                display: 'Select all'
            }, 0);

            /**
             * Override _handleStaticFilter to allow searching by both 'display' and 'value'
             * @param  {String} query The term used to filter list items.
             * TODO: consider extending CUI.Autocomplete instead of overriding its private APIs
             */
            languageSelectorField._handleStaticFilter = function (query) {
                if (query) {
                    this._selectListWidget.filter(function (value, display) {
                        var lcQuery = query.toLowerCase();
                        // "Select all" should always be displayed
                        return value === SELECT_ALL_VALUE || display.toLowerCase().search(lcQuery) !== -1 || value.search(lcQuery) !== -1;
                    });
                } else {
                    this._selectListWidget.filter();
                }
            };

            /**
             * Override _triggerSelected to allow custom "Select all" option handling
             * TODO: consider extending CUI.SelectList instead of overriding its private APIs
             */
            languageSelectorField._selectlist.off('click').on('click', languageSelectorField._selectListWidget._SELECTABLE_SELECTOR,function (event) {
                if($(event.currentTarget).data('value') === SELECT_ALL_VALUE) {
                    languageSelectorField._selectlist.find('.coral-SelectList-item--option:not(.is-hidden):not([data-value="' + SELECT_ALL_VALUE + '"])')
                        .each(function () {
                            var $option = $(this);
                            $option.trigger($.Event('selected', {
                                selectedValue: $option.data('value'),
                                displayedValue: $option.data('display') || $option.text()
                            }));
                        });
                } else {
                    // delegate to the method of super class
                    languageSelectorField._selectListWidget._triggerSelected.call(languageSelectorField._selectListWidget, event);
                }
            });

            // Handle "Deselect all" button click
            $selector.find('.js-language-selector-field-deselect-all').on('click', function (e) {
                e.preventDefault();
                languageSelectorField._deselect(languageSelectorField._tagListWidget.getValues());
            });

            // LISTEN to UpdateLocales event (triggered within the dialog)
            $selector.on('languageSelector:updateLocales', function (e, data) {
                // remove previous selection
                $.each(languageSelectorField._tagListWidget.getValues(), function (idx, item) {
                    languageSelectorField._tagListWidget.removeItem(item);
                });

                // mark provided values as "selected"
                // data.values array of locale codes to be selected (empty array == deselect all)
                $.each(data.values, function (idx, localeId) {
                    var option = findOption(languageSelectorField, localeId);
                    if (!!option) {
                        languageSelectorField._tagListWidget.addItem({
                            display: option.getDisplay(),
                            value: option.getValue()
                        })
                    }
                });
            });

            // LISTEN to ResetLocales event
            $(document).on('languageSelector:resetLocales', function (event, data) {
                var locales = data.locales;
                if (!!locales && !!locales.length && locales.length > 0) {
                    //deselect all previousle selected items
                    languageSelectorField._deselect(languageSelectorField._tagListWidget.getValues());
                    //clear all options
                    languageSelectorField._selectListWidget.clearItems();
                    //init Select all option
                    languageSelectorField._selectListWidget.addOption({
                        value: SELECT_ALL_VALUE,
                        display: 'Select all'
                    }, 0);
                    locales.forEach(function(item, index) {
                        languageSelectorField._selectListWidget.addOption({
                            value: item.code,
                            display: item.name
                        }, index + 1);
                    });
                }
            });
        });
    });

    /************************
     * UTILs
     ************************/

    /**
     * @param localesField instance of CUI.Autocomplete class
     * @param value Option value, as returned by CUI.SelectList.Option.getValue()
     * @returns instance of CUI.SelectList.Option, or undefined
     */
    function findOption(localesField, value) {
        var option = null;
        $.each(localesField._selectListWidget.getItems(), function (idx, item) {
            if (item.getValue() === value) {
                option = item;
                return false; // exit from for-each
            }
        });
        return option;
    }

})(document, Granite.$);